/*
 * Copyright (C) 2006 Red Hat, Inc.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 *
 *   Free Software Foundation
 *   51 Franklin Street, Fifth Floor
 *   Boston, MA  02111-1301
 *   USA.
 */

#ifndef certs_h
#define certs_h

#ident "$Id: certs.h,v 1.8 2006/11/01 16:32:59 nalin Exp $"
#include "pkinit.h"

struct pkinit_principal_name {
	SECItem name_type;
	SECItem **name_string;
};

struct pkinit_krb5_principal_name {
	SECItem realm;
	struct pkinit_principal_name principal_name;
};

struct other_name {
	SECItem oid;
	SECItem data;
};

enum subject_alt_name_type {
	subject_alt_name_other_name = 1,
	subject_alt_name_rfc822_name = 2,
	subject_alt_name_dnsname = 3,
};

struct subject_alt_name {
	int subject_alt_name_type;
	struct {
		struct other_name other_name;
		SECItem rfc822_name;
		SECItem dnsname;
	} subject_alt_name_value;
};

#define	CERT_MATCHES_UPN_SAN		(1 << 0)
#define	CERT_MATCHES_KRB_SAN		(1 << 1)
#define CERT_SAN_MASK			(CERT_MATCHES_UPN_SAN |\
					 CERT_MATCHES_KRB_SAN)
#define	CERT_MATCHES_CLIENT_AUTH_EKU	(1 << 2)
#define	CERT_MATCHES_SERVER_AUTH_EKU	(1 << 3)
#define	CERT_MATCHES_PKINIT_CLIENT_EKU	(1 << 4)
#define	CERT_MATCHES_PKINIT_KDC_EKU	(1 << 5)
#define	CERT_MATCHES_MS_SMARTCARD_EKU	(1 << 6)
#define CERT_EKU_MASK			(CERT_MATCHES_CLIENT_AUTH_EKU |\
					 CERT_MATCHES_PKINIT_CLIENT_EKU |\
					 CERT_MATCHES_PKINIT_KDC_EKU |\
					 CERT_MATCHES_MS_SMARTCARD_EKU)

#define CERT_MATCHES_DC_CERT_TEMPLATE	(1 << 7)
#define CERT_MATCHES_TRUSTED_GUID	(1 << 8)

CERTCertificate *cert_find_preferred_cert(PK11SlotInfo *preferred_slot,
					  struct pkinit_pwcb_args *pwcb_args,
					  krb5_context kcontext,
					  krb5_principal principal,
					  PRInt32 cert_flags);
krb5_error_code cert_validate_client_certificate(CERTCertDBHandle *certdb,
						 CERTCertificate *cert,
						 struct pkinit_pwcb_args *,
						 krb5_context kcontext,
						 krb5_principal client);
krb5_error_code cert_validate_kdc_certificate(CERTCertDBHandle *certdb,
					      CERTCertificate *kdc_cert,
					      struct pkinit_pwcb_args *,
					      krb5_context kcontext,
					      krb5_principal server);

const SECItem *cert_get_oid_dh_public_number(void);
const SECItem *cert_get_oid_dh_key_agreement(void);
const SECItem *cert_get_oid_pkinit_auth_data(void);
const SECItem *cert_get_oid_pkinit_rkey_data(void);
const SECItem *cert_get_oid_pkinit_dhkey_data(void);

SECItem **cert_get_guids(CERTCertificate *cert);

PRBool cert_is_preferred(CERTCertificate *cert,
			 krb5_context kcontext, krb5_principal principal,
			 PRInt32 cert_flags);
PRBool cert_is_domain_controller(CERTCertificate *signer,
				 krb5_context kcontext,
				 krb5_principal tgt_principal);

#endif
