/*
 * Copyright (C) 2006 Red Hat, Inc.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 *
 *   Free Software Foundation
 *   51 Franklin Street, Fifth Floor
 *   Boston, MA  02111-1301
 *   USA.
 */

#ifndef commont_h
#define commont_h

#ident "$Id: commont.h,v 1.7 2006/10/26 19:37:02 nalin Exp $"

#include <krb5.h>
#include <nspr.h>
#include <nss.h>
#include <secasn1t.h>

#include "pkinit.h"

struct algorithm_identifier {
	SECItem algorithm, *parameters;
};

struct subject_public_key_info {
	struct algorithm_identifier algorithm;
	SECItem subject_public_key;
};

struct validation_parms {
	SECItem seed, pgen_counter;
};

struct dh_parameters {
	SECItem p, g;
};

struct domain_parameters {
	SECItem p, g, q, j;
	struct validation_parms *validation_parms;
};

struct issuer_and_serial_number {
	SECItem issuer;
	SECItem serial;
};

struct rc2_cbc_parameters {
	SECItem version;
	SECItem iv;
};

struct des3_cbc_parameters {
	SECItem iv;
};

extern const SEC_ASN1Template common_general_string_template[];
extern const SEC_ASN1Template common_sequence_of_any_template[];

extern const SEC_ASN1Template common_algorithm_identifier_template[];
extern const SEC_ASN1Template common_issuer_and_serial_number_template[];
extern const SEC_ASN1Template common_sequence_of_algorithm_identifier_template[];
extern const SEC_ASN1Template common_subject_public_key_info_template[];
extern const SEC_ASN1Template common_rc2_cbc_parameters_template[];

krb5_data *common_krb5_data_from_secitem(krb5_context kcontext, SECItem *item);
SECItem *common_secitem_from_krb5_data(PLArenaPool *pool, krb5_data *data);

SECItem *common_encode_dh_parameters(PLArenaPool *pool,
				     struct dh_parameters *info);
SECItem *common_encode_domain_parameters(PLArenaPool *pool,
					 struct domain_parameters *info);
SECItem *common_encode_bitstring(PLArenaPool *pool, SECItem *bits);
SECItem *common_encode_integer(PLArenaPool *pool, SECItem *bits);
SECItem *common_encode_subject_public_key_info(PLArenaPool *pool,
					       struct subject_public_key_info *info);
struct dh_parameters *common_decode_dh_parameters(PLArenaPool *pool,
						  SECItem *item);
struct domain_parameters *common_decode_domain_parameters(PLArenaPool *pool,
							  SECItem *item);
SECItem *common_decode_bitstring(PLArenaPool *pool, SECItem *item);
SECItem *common_decode_integer(PLArenaPool *pool, SECItem *item);
struct subject_public_key_info *common_decode_subject_public_key_info(PLArenaPool *pool, SECItem *item);

struct algorithm_identifier **
common_make_algorithm_identifier_list(PLArenaPool *pool,
				      struct algorithm_identifier **list,
				      struct algorithm_identifier *entry);
SECStatus
common_generate_content_encryption_key(PLArenaPool *pool,
				       krb5_context kcontext,
				       SECOidTag preferred_cek,
				       struct algorithm_identifier *algorithm,
				       SECItem *content_enc_key,
				       CK_MECHANISM *mech,
				       int *block_size);

SECItem *common_make_random_item(PLArenaPool *pool,
				 krb5_context kcontext,
				 size_t length);

#endif
