/*
 * Copyright (C) 2006 Red Hat, Inc.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 *
 *   Free Software Foundation
 *   51 Franklin Street, Fifth Floor
 *   Boston, MA  02111-1301
 *   USA.
 */

#ident "$Id: show-cert-guid.c,v 1.1 2006/10/11 18:17:13 nalin Exp $"

#include "../config.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <getopt.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>

#include <nss.h>
#include "commont.h"
#include "bcmst.h"
#include "certs.h"

#if DEBUG
void
pkinit_debug(const char *fmt, ...)
{
	/* Don't do debugging here. */
}
#endif

static SECItem *
item_from_file(PLArenaPool *pool, const char *file)
{
	SECItem *item;
	struct stat st;
	size_t size;
	FILE *fp;

	item = NULL;
	fp = fopen(file, "r");
	if (fp != NULL) {
		if (fstat(fileno(fp), &st) != -1) {
			item = SECITEM_AllocItem(pool, NULL, st.st_size);
			if (item != NULL) {
				size = (size_t) st.st_size;
				if (fread(item->data, 1, size, fp) == size) {
					item->len = st.st_size;
				} else {
					item = NULL;
				}
			}
		}
		fclose(fp);
	}
	return item;
}

int
main(int argc, char **argv)
{
	CERTCertDBHandle *certdb;
	SECItem *certdata, **guids;
	PLArenaPool *pool;
	CERTCertificate *cert;
	int i, j;
	unsigned int k;

	for (i = 1; i < argc; i++) {
		if (argv[i][0] == '-') {
			fprintf(stderr, "Usage: show-cert-guid [file [...]]\n");
			return 1;
		}
	}

	NSS_Init(DEFAULT_PKINIT_CLIENT_DBDIR);

	pool = PORT_NewArena(sizeof(double));
	if (pool == NULL) {
		fprintf(stderr, "Error creating memory pool!\n");
		return 1;
	}

	certdb = CERT_GetDefaultCertDB();
	for (i = 1; i < argc; i++) {
		certdata = item_from_file(pool, argv[i]);
		if (certdata == NULL) {
			fprintf(stderr, "Error reading \"%s\".\n", argv[i]);
			return 1;
		}
		cert = CERT_DecodeCertFromPackage((char *) certdata->data,
						  certdata->len);
		if (cert == NULL) {
			fprintf(stderr, "Error decoding \"%s\".\n", argv[i]);
			return 1;
		}
		guids = cert_get_guids(cert);
		if (guids != NULL) {
			printf("%s\n", cert->subjectName);
			for (j = 0; guids[j] != NULL; j++) {
				printf("\t");
				for (k = 0; k < guids[j]->len; k++) {
					printf("%s%02x", k > 0 ? ":" : "",
					       guids[j]->data[k] & 0xff);
				}
				printf("\n");
			}
		}
	}
	NSS_Shutdown();
	return 0;
}
